package io.skore.smooch_plugin.util

import android.app.Application
import android.content.Context
import android.os.Handler
import android.util.Log
import io.flutter.plugin.common.MethodChannel
import io.skore.smooch_plugin.data.SmoochAuth
import io.skore.smooch_plugin.delegate.ConversationDelegate
import io.skore.smooch_plugin.delegate.MessageModifierDelegate
import io.smooch.core.*
import java.lang.ref.WeakReference

object SmoochUtil {
    private val tag = SmoochUtil::class.java.simpleName

//    private const val FLUTTER_PREFS = "FlutterSharedPreferences"
//    private const val AUTH_FLUTTER_USER_DATA = "flutter.USER"

    private const val SMOOCH_PREFS = "SmoochPluginPreferences"
    private const val AUTH_SMOOCH_APP_ID = "AUTH_SMOOCH_APP_ID"
    private const val AUTH_SMOOCH_USER_ID = "AUTH_SMOOCH_USER_ID"
    private const val AUTH_SMOOCH_TOKEN = "AUTH_SMOOCH_TOKEN"
    private const val AUTH_TOKEN = "AUTH_TOKEN"
    private const val AUTH_EMAIL = "AUTH_EMAIL"
    private const val AUTH_USER_NAME = "AUTH_USER_NAME"
    private const val AUTH_USER_ID = "AUTH_USER_ID"
    private const val AUTH_COMPANY_ID = "AUTH_COMPANY_ID"
    private const val AUTH_REFRESH_TOKEN = "AUTH_REFRESH_TOKEN"
    private const val AUTH_TOKEN_EXPIRATION_DATE = "AUTH_TOKEN_EXPIRATION_DATE"

    var forceInitCallbackSmooch: ((Application, ((Boolean) -> Unit)?) -> Unit)? = null
    var smoochCallback = false
    init {
        forceInitCallbackSmooch = { app, onFinish ->
            Handler(app.mainLooper).postDelayed({
                val success = Smooch.getInitializationStatus() == InitializationStatus.Success
                if (!smoochCallback && success && Smooch.getLastLoginResult() == null)
                    onFinish?.invoke(success)
                else if (!success && Smooch.getLastLoginResult() == null) {
                    forceInitCallbackSmooch?.invoke(app, onFinish)
                }
            }, 3000)
        }
    }
    var methodChannelRef: WeakReference<MethodChannel?>? = null

    private fun init(app: Application, appId: String, onFinish: ((Boolean) -> Unit)? = null) {
        Log.d(tag, "Smooch init app ID: $appId")
        val initStatus = Smooch.getInitializationStatus()
        val changedAppId = Smooch.getSettings()?.appId != appId;
        if (initStatus != InitializationStatus.Success || changedAppId) {
            smoochCallback = false
            Smooch.init(app, Settings(appId)) { initResponse ->
                smoochCallback = true
                Log.d(tag, "Smooch init status: ${initResponse.status}")
                onFinish?.invoke(initResponse.status == 200)
            }
            forceInitCallbackSmooch?.invoke(app, onFinish)
        } else {
            Log.d(tag, "Smooch already initialized.")
            onFinish?.invoke(true)
        }
    }

    fun login(app: Application, smoochAuth: SmoochAuth, onFinish: ((Boolean) -> Unit)? = null) = init(app, smoochAuth.appId) { isInit ->
        if (!isInit) {
            Log.d(tag, "Failed to initialize smooch. Login can not proceed.")
            onFinish?.invoke(false)
            return@init
        }

        val loginResult = Smooch.getLastLoginResult()
        if (loginResult != LoginResult.Success) {
            Smooch.login(smoochAuth.userId, smoochAuth.token) { loginResponse ->
                Log.d(tag, "Smooch login status: ${loginResponse.status} for ${smoochAuth.userId}/${smoochAuth.token}")
                onFinish?.invoke(loginResponse.status == 200 || loginResponse.status == 204)
            }
        } else {
            Log.d(tag, "User already logged in.")
            onFinish?.invoke(true)
        }
    }

    private fun getUserProp(context: Context) = context.getSharedPreferences(SMOOCH_PREFS, Context.MODE_PRIVATE)?.run {
        val userProp = HashMap<String, Any>()
        getString(AUTH_TOKEN, null)?.let {
            userProp["skrTk"] = it
            userProp["skoreToken"] = it
        }
        getString(AUTH_EMAIL, null)?.let {
            userProp["userEmail"] = it
        }
        getString(AUTH_USER_NAME, null)?.let {
            userProp["userName"] = it
        }
        if (contains(AUTH_USER_ID)) {
            userProp["userId"] = getInt(AUTH_USER_ID, 0)
        }
        if (contains(AUTH_COMPANY_ID)) {
            userProp["companyId"] = getInt(AUTH_COMPANY_ID, 0)
        }
        getString(AUTH_REFRESH_TOKEN, null)?.let {
            userProp["refreshToken"] = it
        }
        if (contains(AUTH_TOKEN_EXPIRATION_DATE)) {
            userProp["tokenExpirationDate"] = getLong(AUTH_TOKEN_EXPIRATION_DATE, 0)
        }
        if (userProp.isEmpty()) null else userProp
    }

    fun addDelegates(context: Context, userProp: Map<String, Any>? = null) {
        Smooch.getConversation()?.apply {
            (userProp ?: getUserProp(context))?.let {
                User.getCurrentUser().addProperties(it)
            } ?: run {
                Log.e(tag, "Missing user properties.")
            }

            methodChannelRef?.get()?.let {
                delegate = ConversationDelegate(it)
            } ?: run {
                Log.e(tag, "Method channel is not available.")
            }
            messageModifierDelegate = MessageModifierDelegate()
        }
    }

    fun setAuth(context: Context, smoochAuth: SmoochAuth?) {
        context.getSharedPreferences(SMOOCH_PREFS, Context.MODE_PRIVATE)?.run {
            edit()?.apply {
                putString(AUTH_SMOOCH_APP_ID, smoochAuth?.appId)
                putString(AUTH_SMOOCH_USER_ID, smoochAuth?.userId)
                putString(AUTH_SMOOCH_TOKEN, smoochAuth?.token)
                smoochAuth?.userProp.let { userProp ->
                    putString(AUTH_TOKEN, userProp?.get("skrTk") as String?)
                    putString(AUTH_EMAIL, userProp?.get("userEmail") as String?)
                    putString(AUTH_USER_NAME, userProp?.get("userName") as String?)
                    putString(AUTH_REFRESH_TOKEN, userProp?.get("refreshToken") as String?)
                    (userProp?.get("userId") as Int?).let {
                        if (it == null) {
                            remove(AUTH_USER_ID)
                        } else {
                            putInt(AUTH_USER_ID, it)
                        }
                    }
                    (userProp?.get("companyId") as Int?).let {
                        if (it == null) {
                            remove(AUTH_COMPANY_ID)
                        } else {
                            putInt(AUTH_COMPANY_ID, it)
                        }
                    }
                    (userProp?.get("tokenExpirationDate") as Long?).let {
                        if (it == null) {
                            remove(AUTH_TOKEN_EXPIRATION_DATE)
                        } else {
                            putLong(AUTH_TOKEN_EXPIRATION_DATE, it)
                        }
                    }
                }
                apply()
            }
        }
    }

    fun getAuth(context: Context) = context.getSharedPreferences(SMOOCH_PREFS, Context.MODE_PRIVATE)?.run {
        val appId = getString(AUTH_SMOOCH_APP_ID, null)
        val userId = getString(AUTH_SMOOCH_USER_ID, null)
        val token = getString(AUTH_SMOOCH_TOKEN, null)
        if (appId.isNullOrBlank() || userId.isNullOrBlank() || token.isNullOrBlank()) null else SmoochAuth(appId, userId, token)
    }
}